@@ -1,5 +1,39 @@
 #if 0
 
+N_("Bourbon Pork Loin")
+N_("This is a modified version of a recipe found on a website.")
+N_("1. Combine the spice rub ingredients, and rub all over the pork. Let sit at room temperature as you preheat your oven to 110ºC.")
+N_("2. Place the pork on a skillet, then place in the oven and roast the pork for about 35 minutes. As the pork roasts, combine the glaze ingredients in a small saucepan; bring to a simmer over medium-low heat and simmer until dark and thickened, about 15 minutes. Set the glaze aside.")
+N_("3. Remove the cooked pork and set on a cutting board to rest; loosely tent with tin foil. Increase the oven temperature to 260ºC. As the oven heats, place the skillet on the stove top, and add the oil, onion, and a few sprinkles of salt; cook until the onions are softened. Be careful because once onions start to caramelize they will quickly burn. Cover the pork liberally with glaze.")
+N_("4. Roast a little bit more, less than 5 minutes. Let it rest and serve with smashed potatoes and vegs.")
+N_("** If you decide to add apples, they go at the end of the 3rd step, before dripping glaze on the meat.")
+N_("Some use maple syrup but I prefer the taste of pure honey mixed with the flavour of pork meat.")
+N_("Same for apples; as some people add one or two. I don't like them but you if you fancy, feel free to add them.")
+N_("Also, using lard makes this dish very heavy but using olive oil is pointless as its flavour will be missed. But if you have nothing else, use them. It won't hurt.")
+N_("The Glaze")
+N_("The Glaze")
+N_("The Glaze")
+N_("The Glaze")
+N_("The Glaze")
+N_("The Spice Cover")
+N_("The Spice Cover")
+N_("The Spice Cover")
+N_("The Spice Cover")
+N_("The Meat")
+N_("The Meat")
+N_("Others")
+N_("Others")
+N_("Others")
+N_("Henry's original stew")
+N_("My friend Heinrich made up this stew a winter Saturday.")
+N_("It can be eaten with rice or alone.")
+N_("Cook the meat with the species, not completely, just enough to get it toasted and mix the flavours. ")
+N_("Add the vegetables and beans.  Cook a bit more.  ")
+N_("Add abundant broth, cover the pot and let it cook on slow fire until is done. ")
+N_("Can be served with rice  (like in the picture)  or alone.")
+N_("Beans can be of any kind, and can be bought canned, in which case would be a can of each kind.")
+N_("Juniper berries and Bay leaves are optional.")
+N_(" ")
 N_("Pesce alla sale")
 N_("It's an easy but tasty way to prepare fish.")
 N_("Despite all the salt, the result isn't salty but soft and nice.")
