@@ -91,8 +91,6 @@ struct _GrDetailsPage
         GtkWidget *error_revealer;
 
         guint save_timeout;
-
-        char *uri;
 };
 
 G_DEFINE_TYPE (GrDetailsPage, gr_details_page, GTK_TYPE_BOX)
@@ -221,6 +219,7 @@ shop_it (GrDetailsPage *page)
 }
 
 static gboolean save_notes (gpointer data);
+static void details_page_reload (GrDetailsPage *page, GrRecipe *recipe);
 
 static void
 details_page_finalize (GObject *object)
@@ -238,7 +237,6 @@ details_page_finalize (GObject *object)
         g_clear_object (&self->ingredients);
         g_clear_object (&self->printer);
         g_clear_object (&self->exporter);
-        g_clear_pointer (&self->uri, g_free);
         g_clear_pointer (&self->ing_text, g_free);
 
         G_OBJECT_CLASS (gr_details_page_parent_class)->finalize (object);
@@ -270,9 +268,12 @@ save_notes (gpointer data)
         g_object_set (page->recipe, "notes", text, NULL);
 
         store = gr_recipe_store_get ();
+
+        g_signal_handlers_block_by_func (store, details_page_reload, page);
         if (!gr_recipe_store_update_recipe (store, page->recipe, id, &error)) {
                 g_warning ("Error: %s", error->message);
         }
+        g_signal_handlers_unblock_by_func (store, details_page_reload, page);
 
 out:
         page->save_timeout = 0;
@@ -292,14 +293,10 @@ schedule_save (GtkTextBuffer *buffer, GrDetailsPage *page)
 }
 
 static gboolean
-activate_uri_at_idle (gpointer data)
+activate_link (GtkLabel      *label,
+               const char    *uri,
+               GrDetailsPage *page)
 {
-        GrDetailsPage *page = data;
-        g_autofree char *uri = NULL;
-
-        uri = page->uri;
-        page->uri = NULL;
-
         if (g_str_has_prefix (uri, "image:")) {
                 int idx;
 
@@ -328,20 +325,6 @@ activate_uri_at_idle (gpointer data)
                 }
         }
 
-        return G_SOURCE_REMOVE;
-}
-
-static gboolean
-activate_link (GtkLabel      *label,
-               const char    *uri,
-               GrDetailsPage *page)
-{
-        g_free (page->uri);
-        page->uri = g_strdup (uri);
-
-        // FIXME: We can avoid the idle with GTK+ 3.22.6 or newer
-        g_idle_add (activate_uri_at_idle, page);
-
         return TRUE;
 }
 
@@ -471,6 +454,8 @@ populate_ingredients (GrDetailsPage *page,
                                      "title", segments[j],
                                      "editable-title", FALSE,
                                      "editable", FALSE,
+                                     "scale-num", num,
+                                     "scale-denom", denom,
                                      "ingredients", page->ing_text,
                                      NULL);
                 gtk_container_add (GTK_CONTAINER (page->ingredients_box), list);
@@ -574,8 +559,6 @@ void
 gr_details_page_set_recipe (GrDetailsPage *page,
                             GrRecipe      *recipe)
 {
-        const char *id = NULL;
-        const char *old_id = NULL;
         const char *author;
         const char *prep_time;
         const char *cook_time;
@@ -583,7 +566,6 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         const char *meal;
         const char *season;
         int serves;
-        int want_serves;
         const char *ingredients;
         const char *instructions;
         const char *notes;
@@ -593,15 +575,9 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         g_autoptr(GrIngredientsList) ing = NULL;
         GPtrArray *images;
         gboolean favorite;
-        gboolean same_recipe;
         int index;
         g_autofree char *processed = NULL;
 
-        if (page->recipe)
-                old_id = gr_recipe_get_id (page->recipe);
-        id = gr_recipe_get_id (recipe);
-        same_recipe = g_strcmp0 (old_id, id) == 0;
-
         g_set_object (&page->recipe, recipe);
 
         author = gr_recipe_get_author (recipe);
@@ -615,10 +591,7 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         notes = gr_recipe_get_translated_notes (recipe);
         instructions = gr_recipe_get_translated_instructions (recipe);
         description = gr_recipe_get_translated_description (recipe);
-        if (same_recipe)
-                g_object_get (page->recipe_image, "index", &index, NULL);
-        else
-                index = gr_recipe_get_default_image (recipe);
+        index = gr_recipe_get_default_image (recipe);
 
         images = gr_recipe_get_images (recipe);
         gr_image_viewer_set_images (GR_IMAGE_VIEWER (page->recipe_image), images, index);
@@ -628,12 +601,7 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         g_free (page->ing_text);
         page->ing_text = g_strdup (ingredients);
 
-        if (same_recipe)
-                want_serves = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (page->serves_spin));
-        else
-                want_serves = serves;
-
-        populate_ingredients (page, want_serves, serves);
+        populate_ingredients (page, serves, serves);
 
         if (prep_time[0] == '\0') {
                 gtk_widget_hide (page->prep_time_label);
@@ -689,7 +657,7 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         gtk_label_set_label (GTK_LABEL (page->instructions_label), processed);
         gtk_label_set_track_visited_links (GTK_LABEL (page->instructions_label), FALSE);
 
-        gtk_spin_button_set_value (GTK_SPIN_BUTTON (page->serves_spin), want_serves);
+        gtk_spin_button_set_value (GTK_SPIN_BUTTON (page->serves_spin), serves);
         gtk_widget_set_sensitive (page->serves_spin, ing != NULL);
 
         store = gr_recipe_store_get ();
