@@ -22,6 +22,7 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <stdlib.h>
 
 #include "gr-ingredients-viewer-row.h"
 #include "gr-ingredients-viewer.h"
@@ -31,7 +32,6 @@
 #include "gr-number.h"
 #include "gr-recipe-store.h"
 
-
 struct _GrIngredientsViewerRow
 {
         GtkListBoxRow parent_instance;
@@ -47,6 +47,8 @@ struct _GrIngredientsViewerRow
         GtkWidget *unit_event_box;
         GtkWidget *ingredient_event_box;
         GtkWidget *unit_help_popover;
+        GtkEntryCompletion *unit_completion;
+        GtkCellRenderer *unit_cell;
 
         char *amount;
         char *unit;
@@ -146,7 +148,7 @@ update_unit (GrIngredientsViewerRow *row)
         space = amount[0] ? " " : "";
         unit = row->unit ? row->unit : "";
         tmp = g_strdup_printf ("%s%s%s", amount, space, unit);
-        if (tmp[0] == '\0') {
+        if (tmp[0] == '\0' && row->editable) {
                 gtk_style_context_add_class (gtk_widget_get_style_context (row->unit_label), "dim-label");
                 gtk_label_set_label (GTK_LABEL (row->unit_label), _("Amount…"));
         }
@@ -280,14 +282,8 @@ emit_delete (GrIngredientsViewerRow *row)
 static void
 edit_ingredient (GrIngredientsViewerRow *row)
 {
-        GrIngredientsViewer *viewer;
-
-        viewer = GR_INGREDIENTS_VIEWER (gtk_widget_get_ancestor (GTK_WIDGET (row), GR_TYPE_INGREDIENTS_VIEWER));
-
         if (row->editable) {
                 save_unit (row);
-
-                set_active_row (viewer, GTK_WIDGET (row));
                 gtk_entry_set_text (GTK_ENTRY (row->ingredient_entry), row->ingredient);
                 gtk_stack_set_visible_child_name (GTK_STACK (row->ingredient_stack), "ingredient_entry");
                 gtk_widget_grab_focus (row->ingredient_entry);
@@ -313,19 +309,15 @@ edit_unit (GrIngredientsViewerRow *row)
         const char *amount;
         const char *space;
         const char *unit;
-        GrIngredientsViewer *viewer;
 
         amount = row->amount ? row->amount : "";
         space = amount[0] ? " " : "";
         unit = row->unit ? row->unit : "";
         tmp = g_strdup_printf ("%s%s%s", amount, space, unit);
 
-        viewer = GR_INGREDIENTS_VIEWER (gtk_widget_get_ancestor (GTK_WIDGET (row), GR_TYPE_INGREDIENTS_VIEWER));
-
         save_ingredient (row);
 
         if (row->editable) {
-                set_active_row (viewer, GTK_WIDGET (row));
                 gtk_entry_set_text (GTK_ENTRY (row->unit_entry), tmp);
                 gtk_stack_set_visible_child_name (GTK_STACK (row->unit_stack), "unit_entry");
                 gtk_widget_grab_focus (row->unit_entry);
@@ -351,6 +343,7 @@ parse_unit (const char  *text,
         char *tmp;
         const char *str;
         GrNumber number;
+        g_autofree char *num = NULL;
 
         g_clear_pointer (amount, g_free);
         g_clear_pointer (unit, g_free);
@@ -363,7 +356,7 @@ parse_unit (const char  *text,
                 return FALSE;
         }
 
-        *amount = g_strndup (str, tmp - str);
+        *amount = gr_number_format (&number);
         skip_whitespace (&tmp);
         if (tmp)
                 *unit = g_strdup (tmp);
@@ -401,8 +394,12 @@ save_unit (GrIngredientsViewerRow *row)
 
         visible = gtk_stack_get_visible_child (GTK_STACK (row->unit_stack));
         if (visible == row->unit_entry) {
-                if (!parse_unit (gtk_entry_get_text (GTK_ENTRY (row->unit_entry)), &row->amount, &row->unit)) {
-                        set_unit_error (row, TRUE);
+                const char *text;
+
+                text = gtk_entry_get_text (GTK_ENTRY (row->unit_entry));
+                if (!parse_unit (text, &row->amount, &row->unit)) {
+                        if (text && text[strspn (text, " ")])
+                                set_unit_error (row, TRUE);
                 }
 
                 update_unit (row);
@@ -576,6 +573,7 @@ drag_begin (GtkWidget      *widget,
         cairo_t *cr;
         GtkWidget *row;
         int x, y;
+        GtkWidget *viewer;
 
         row = gtk_widget_get_ancestor (widget, GTK_TYPE_LIST_BOX_ROW);
 
@@ -583,9 +581,9 @@ drag_begin (GtkWidget      *widget,
         surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, alloc.width, alloc.height);
         cr = cairo_create (surface);
 
-        gtk_style_context_add_class (gtk_widget_get_style_context (row), "during-dnd");
+        gtk_style_context_add_class (gtk_widget_get_style_context (row), "drag-icon");
         gtk_widget_draw (row, cr);
-        gtk_style_context_remove_class (gtk_widget_get_style_context (row), "during-dnd");
+        gtk_style_context_remove_class (gtk_widget_get_style_context (row), "drag-icon");
 
         gtk_widget_translate_coordinates (widget, row, 0, 0, &x, &y);
         cairo_surface_set_device_offset (surface, -x, -y);
@@ -593,6 +591,20 @@ drag_begin (GtkWidget      *widget,
 
         cairo_destroy (cr);
         cairo_surface_destroy (surface);
+
+        viewer = gtk_widget_get_ancestor (row, GR_TYPE_INGREDIENTS_VIEWER);
+        gr_ingredients_viewer_set_drag_row (GR_INGREDIENTS_VIEWER (viewer), row);
+}
+
+static void
+drag_end (GtkWidget      *widget,
+          GdkDragContext *context,
+          gpointer        data)
+{
+        GtkWidget *viewer;
+
+        viewer = gtk_widget_get_ancestor (widget, GR_TYPE_INGREDIENTS_VIEWER);
+        gr_ingredients_viewer_set_drag_row (GR_INGREDIENTS_VIEWER (viewer), NULL);
 }
 
 void
@@ -610,31 +622,6 @@ drag_data_get (GtkWidget        *widget,
                                 sizeof (gpointer));
 }
 
-static void
-drag_data_received (GtkWidget        *widget,
-                    GdkDragContext   *context,
-                    gint              x,
-                    gint              y,
-                    GtkSelectionData *selection_data,
-                    guint             info,
-                    guint32           time,
-                    gpointer          data)
-{
-        GtkWidget *target;
-        GtkWidget *row;
-        GtkWidget *source;
-
-        target = gtk_widget_get_ancestor (widget, GTK_TYPE_LIST_BOX_ROW);
-
-        row = (gpointer)* (gpointer*)gtk_selection_data_get_data (selection_data);
-        source = gtk_widget_get_ancestor (row, GTK_TYPE_LIST_BOX_ROW);
-
-        if (target == source)
-                return;
-
-        g_signal_emit (source, signals[MOVE], 0, target);
-}
-
 static int
 sort_func (GtkTreeModel *model,
            GtkTreeIter  *a,
@@ -696,69 +683,65 @@ get_ingredients_model (void)
         return GTK_TREE_MODEL (g_object_ref (ingredients_model));
 }
 
-static void
-prepend_amount (GtkTreeModel *model,
-                GtkTreeIter  *iter,
-                GValue       *value,
-                gint          column,
-                gpointer      data)
-{
-        GrIngredientsViewerRow *row = data;
-        GtkTreeModelFilter *filter_model = GTK_TREE_MODEL_FILTER (model);
-        GtkTreeModel *child_model;
-        GtkTreeIter child_iter;
-        g_autofree char *amount = NULL;
-        g_autofree char *tmp = NULL;
-        g_autofree char *unit = NULL;
-        g_autofree char *new_unit = NULL;
-
-        parse_unit (gtk_entry_get_text (GTK_ENTRY (row->unit_entry)), &amount, &unit);
-
-        child_model = gtk_tree_model_filter_get_model (filter_model);
-        gtk_tree_model_filter_convert_iter_to_child_iter (filter_model, &child_iter, iter);
-
-        gtk_tree_model_get (child_model, &child_iter, column, &new_unit, -1);
-
-        tmp = g_strdup_printf ("%s %s", amount, new_unit);
-        g_value_set_string (value, tmp);
-}
-
 static GtkTreeModel *
 get_units_model (GrIngredientsViewerRow *row)
 {
         static GtkListStore *store = NULL;
-        GtkTreeModel *model;
-        GType types[1];
 
         if (store == NULL) {
                 const char **names;
                 int i;
 
-                store = gtk_list_store_new (1, G_TYPE_STRING);
+                store = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
                 gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (store),
                                                          sort_func, NULL, NULL);
                 gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
                                                       GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
                                                       GTK_SORT_ASCENDING);
 
+                gtk_list_store_insert_with_values (store, NULL, -1,
+                                                   0, "",
+                                                   1, "",
+                                                   2, "",
+                                                   3, "",
+                                                   4, "",
+                                                   -1);
+
                 names = gr_unit_get_names ();
                 for (i = 0; names[i]; i++) {
+                        const char *abbrev;
+                        const char *name;
+                        const char *plural;
+
+                        g_autofree char *tmp = NULL;
+                        g_autofree char *tmp2 = NULL;
+
+                        abbrev = gr_unit_get_abbreviation (names[i]);
+                        name = gr_unit_get_display_name (names[i]);
+                        plural = gr_unit_get_plural (names[i]);
+
+
+                        if (strcmp (abbrev, name) == 0)
+                                tmp = g_strdup (name);
+                        else
+                                tmp = g_strdup_printf ("%s (%s)", name, abbrev);
+
+                        if (strcmp (abbrev, plural) == 0)
+                                tmp2 = g_strdup (name);
+                        else 
+                                tmp2 = g_strdup_printf ("%s (%s)", plural, abbrev);
+
                         gtk_list_store_insert_with_values (store, NULL, -1,
-                                                           0, names[i],
+                                                           0, abbrev,
+                                                           1, name,
+                                                           2, tmp,
+                                                           3, plural,
+                                                           4, tmp2,
                                                            -1);
                 }
         }
 
-        model = gtk_tree_model_filter_new (GTK_TREE_MODEL (store), NULL);
-        types[0] = G_TYPE_STRING;
-        gtk_tree_model_filter_set_modify_func (GTK_TREE_MODEL_FILTER (model),
-                                               G_N_ELEMENTS (types),
-                                               types,
-                                               prepend_amount,
-                                               row,
-                                               NULL);
-
-        return model;
+        return g_object_ref (store);
 }
 
 static void
@@ -768,6 +751,92 @@ gr_ingredients_viewer_row_init (GrIngredientsViewerRow *self)
         gtk_widget_init_template (GTK_WIDGET (self));
 }
 
+static void
+get_amount (GtkCellLayout   *layout,
+            GtkCellRenderer *renderer,
+            GtkTreeModel    *model,
+            GtkTreeIter     *iter,
+            gpointer         data)
+{
+        GrIngredientsViewerRow *row = data;
+        g_autofree char *amount = NULL;
+        g_autofree char *unit = NULL;
+        parse_unit (gtk_entry_get_text (GTK_ENTRY (row->unit_entry)), &amount, &unit);
+        g_object_set (renderer, "text", amount, NULL);
+}
+
+static gboolean
+match_func (GtkEntryCompletion *completion,
+            const char         *key,
+            GtkTreeIter        *iter,
+            gpointer            data)
+{
+        GrIngredientsViewerRow *row = data;
+        GtkTreeModel *model;
+        g_autofree char *abbrev = NULL;
+        g_autofree char *name = NULL;
+        g_autofree char *amount = NULL;
+        g_autofree char *unit = NULL;
+        g_autofree char *plural = NULL;
+
+        model = gtk_entry_completion_get_model (completion);
+        gtk_tree_model_get (model, iter, 0, &abbrev, 1, &name, 3, &plural, -1);
+
+        parse_unit (gtk_entry_get_text (GTK_ENTRY (row->unit_entry)), &amount, &unit);
+
+        if (!amount || !unit)
+                return FALSE;
+
+        if (g_str_has_prefix (abbrev, unit) ||
+            g_str_has_prefix (name, unit) ||
+            g_str_has_prefix (plural, unit))
+                return TRUE;
+
+        return FALSE;
+}
+
+static gboolean
+match_selected (GtkEntryCompletion *completion,
+                GtkTreeModel       *model,
+                GtkTreeIter        *iter,
+                gpointer            data)
+{
+        GrIngredientsViewerRow *row = data;
+        g_autofree char *abbrev = NULL;
+        g_autofree char *amount = NULL;
+        g_autofree char *unit = NULL;
+        g_autofree char *tmp = NULL;
+        g_autofree char *plural = NULL;
+
+        gtk_tree_model_get (model, iter, 0, &abbrev, -1);
+
+        parse_unit (gtk_entry_get_text (GTK_ENTRY (row->unit_entry)), &amount, &unit);
+
+        tmp = g_strdup_printf ("%s %s", amount, abbrev);
+
+        gtk_entry_set_text (GTK_ENTRY (row->unit_entry), tmp);
+
+        return TRUE;
+}
+
+static void
+text_changed (GObject    *object,
+              GParamSpec *pspec,
+              gpointer    data)
+{
+        GtkEntry *entry = GTK_ENTRY (object);
+        GrIngredientsViewerRow *row = data;
+        GrNumber number;
+        char *text;
+
+        text = (char *) gtk_entry_get_text (entry);
+        gr_number_parse (&number, &text, NULL);
+        if (number.value > 1)
+                gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (row->unit_completion), row->unit_cell, "text", 4, NULL);
+        else
+                gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (row->unit_completion), row->unit_cell, "text", 2, NULL);
+}
+
 static void
 setup_editable_row (GrIngredientsViewerRow *self)
 {
@@ -777,24 +846,41 @@ setup_editable_row (GrIngredientsViewerRow *self)
                 g_autoptr(GtkEntryCompletion) completion = NULL;
                 g_autoptr(GtkTreeModel) ingredients_model = NULL;
                 g_autoptr(GtkTreeModel) units_model = NULL;
+                //PangoAttrList *attrs = NULL;
+                GtkCellRenderer *cell = NULL;
+
+                completion = gtk_entry_completion_new ();
+
+
 
                 gtk_drag_source_set (self->drag_handle, GDK_BUTTON1_MASK, entries, 1, GDK_ACTION_MOVE);
                 g_signal_connect (self->drag_handle, "drag-begin", G_CALLBACK (drag_begin), NULL);
+                g_signal_connect (self->drag_handle, "drag-end", G_CALLBACK (drag_end), NULL);
                 g_signal_connect (self->drag_handle, "drag-data-get", G_CALLBACK (drag_data_get), NULL);
 
-                gtk_drag_dest_set (GTK_WIDGET (self), GTK_DEST_DEFAULT_ALL, entries, 1, GDK_ACTION_MOVE);
-                g_signal_connect (self, "drag-data-received", G_CALLBACK (drag_data_received), NULL);
-
                 ingredients_model = get_ingredients_model ();
-                completion = gtk_entry_completion_new ();
                 gtk_entry_completion_set_model (completion, ingredients_model);
                 gtk_entry_completion_set_text_column (completion, 0);
                 gtk_entry_set_completion (GTK_ENTRY (self->ingredient_entry), completion);
 
+
                 units_model = get_units_model (self);
                 completion = gtk_entry_completion_new ();
                 gtk_entry_completion_set_model (completion, units_model);
-                gtk_entry_completion_set_text_column (completion, 0);
+                g_object_set (completion, "text-column", 2, NULL);
+
+                cell = gtk_cell_renderer_text_new ();
+                gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (completion), cell, get_amount, self, NULL);
+                gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion), cell, FALSE);
+
+                cell = gtk_cell_renderer_text_new ();
+                gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion), cell, TRUE);
+                self->unit_cell = cell;
+                self->unit_completion = completion;
+                g_signal_connect (self->unit_entry, "notify::text", G_CALLBACK (text_changed), self);
+
+                gtk_entry_completion_set_match_func (completion, match_func, self, NULL);
+                g_signal_connect (completion, "match-selected", G_CALLBACK (match_selected), self);
                 gtk_entry_set_completion (GTK_ENTRY (self->unit_entry), completion);
         }
         else {
@@ -802,9 +888,6 @@ setup_editable_row (GrIngredientsViewerRow *self)
                 g_signal_handlers_disconnect_by_func (self->drag_handle, drag_begin, NULL);
                 g_signal_handlers_disconnect_by_func (self->drag_handle, drag_data_get, NULL);
 
-                gtk_drag_dest_unset (GTK_WIDGET (self));
-                g_signal_handlers_disconnect_by_func (self, drag_data_received, NULL);
-
                 gtk_entry_set_completion (GTK_ENTRY (self->ingredient_entry), NULL);
                 gtk_entry_set_completion (GTK_ENTRY (self->unit_entry), NULL);
         }
@@ -813,7 +896,7 @@ setup_editable_row (GrIngredientsViewerRow *self)
 GtkWidget *
 gr_ingredients_viewer_row_has_error (GrIngredientsViewerRow *row)
 {
-        if (row->amount == NULL || row->amount[0] == '\0')
+        if (gtk_style_context_has_class (gtk_widget_get_style_context (row->unit_label), "error"))
                 return gtk_widget_get_parent (row->unit_label);
 
         if (row->ingredient == NULL || row->ingredient[0] == '\0')
