@@ -735,7 +735,6 @@ populate_recipe_list (GrRecipeExporter *exporter,
 {
         GList *l;
 
-        gtk_list_box_set_header_func (GTK_LIST_BOX (list), all_headers, NULL, NULL);
         gtk_list_box_set_sort_func (GTK_LIST_BOX (list), sort_recipe_row, NULL, NULL);
 
         g_signal_connect (list, "row-activated", G_CALLBACK (row_activated), exporter);
@@ -791,7 +790,8 @@ gr_recipe_exporter_export (GrRecipeExporter *exporter,
 
         for (i = 0; keys[i]; i++) {
                 g_autoptr(GrRecipe) r = gr_recipe_store_get_recipe (store, keys[i]);
-                exporter->recipes = g_list_prepend (exporter->recipes, g_object_ref (r));
+                if (r)
+                        exporter->recipes = g_list_prepend (exporter->recipes, g_object_ref (r));
         }
 
         show_export_dialog (exporter);
