@@ -28,6 +28,9 @@
 #include "gr-image.h"
 #include "gr-chef.h"
 #include "gr-recipe-store.h"
+#include "gr-cuisine.h"
+#include "gr-meal.h"
+#include "gr-season.h"
 #include "gr-utils.h"
 
 
@@ -120,6 +123,9 @@ begin_print (GtkPrintOperation *operation,
         g_autofree char *instructions = NULL;
         g_autoptr(GrChef) chef = NULL;
         GrRecipeStore *store;
+        PangoLayout *layout;
+        int amount_width;
+        const char *value;
 
         store = gr_recipe_store_get ();
         chef = gr_recipe_store_get_chef (store, gr_recipe_get_author (printer->recipe));
@@ -157,29 +163,80 @@ begin_print (GtkPrintOperation *operation,
         g_string_append_printf (s, "%s %s\n", _("Preparation:"), gr_recipe_get_prep_time (printer->recipe));
         g_string_append_printf (s, "%s %s\n", _("Cooking:"), gr_recipe_get_cook_time (printer->recipe));
         g_string_append_printf (s, "%s %d\n", _("Serves:"), gr_recipe_get_serves (printer->recipe));
+        value = gr_recipe_get_cuisine (printer->recipe);
+        if (value && *value) {
+                const char *title;
+                gr_cuisine_get_data (value, &title, NULL, NULL);
+                g_string_append_printf (s, "%s %s\n", _("Cuisine:"), title);
+        }
+        value = gr_recipe_get_category (printer->recipe);
+        if (value && *value) {
+                g_string_append_printf (s, "%s %s\n", _("Meal:"), gr_meal_get_title (value));
+        }
+        value = gr_recipe_get_season (printer->recipe);
+        if (value && *value) {
+                g_string_append_printf (s, "%s %s\n", _("Season:"), gr_season_get_title (value));
+        }
+
         g_string_append (s, "\n");
 
         pango_layout_set_text (printer->left_layout, s->str, s->len);
 
         g_string_truncate (s, 0);
 
+        ingredients = gr_ingredients_list_new (gr_recipe_get_ingredients (printer->recipe));
+        segs = gr_ingredients_list_get_segments (ingredients);
+
+        layout = gtk_print_context_create_pango_layout (context);
+        pango_layout_set_width (layout, width * PANGO_SCALE);
+        pango_layout_set_font_description (layout, body_font);
+
+        for (j = 0; segs[j]; j++) {
+                ings = gr_ingredients_list_get_ingredients (ingredients, segs[j]);
+                for (i = 0; ings[i]; i++) {
+                        g_autofree char *unit = NULL;
+
+                        unit = gr_ingredients_list_scale_unit (ingredients, segs[j], ings[i], 1, 1);
+                        g_string_append (s, unit);
+                        g_string_append (s, " \n");
+                }
+        }
+        pango_layout_set_text (layout, s->str, s->len);
+        pango_layout_get_size (layout, &amount_width, NULL);
+        g_clear_object (&layout);
+
+        g_string_truncate (s, 0);
+
         printer->bottom_layout = gtk_print_context_create_pango_layout (context);
         pango_layout_set_width (printer->bottom_layout, width * PANGO_SCALE);
         pango_layout_set_font_description (printer->bottom_layout, body_font);
 
-        tabs = pango_tab_array_new (2, FALSE);
-        pango_tab_array_set_tab (tabs, 0, PANGO_TAB_LEFT, 0);
-        pango_tab_array_set_tab (tabs, 1, PANGO_TAB_LEFT, (width / 2) * PANGO_SCALE);
-        pango_layout_set_tabs (printer->bottom_layout, tabs);
-        pango_tab_array_free (tabs);
-
         g_string_append (s, gr_recipe_get_translated_description (printer->recipe));
         g_string_append (s, "\n\n");
 
         attrs = pango_attr_list_new ();
 
-        ingredients = gr_ingredients_list_new (gr_recipe_get_ingredients (printer->recipe));
-        segs = gr_ingredients_list_get_segments (ingredients);
+        value = gr_recipe_get_notes (printer->recipe);
+        if (value && *value) {
+                attr = pango_attr_font_desc_new (title_font);
+                attr->start_index = s->len;
+                g_string_append (s, _("Notes"));
+                attr->end_index = s->len + 1;
+                pango_attr_list_insert (attrs, attr);
+
+                g_string_append (s, "\n");
+                g_string_append (s, value);
+                g_string_append (s, "\n\n");
+        }
+
+        tabs = pango_tab_array_new (2, FALSE);
+        pango_tab_array_set_tab (tabs, 0, PANGO_TAB_LEFT, 0);
+        pango_tab_array_set_tab (tabs, 1, PANGO_TAB_LEFT, amount_width);
+        pango_tab_array_set_tab (tabs, 2, PANGO_TAB_LEFT, (width / 2) * PANGO_SCALE);
+        pango_tab_array_set_tab (tabs, 3, PANGO_TAB_LEFT, (width / 2) * PANGO_SCALE + amount_width);
+        pango_layout_set_tabs (printer->bottom_layout, tabs);
+        pango_tab_array_free (tabs);
+
         for (j = 0; segs[j]; j++) {
                 attr = pango_attr_font_desc_new (title_font);
                 attr->start_index = s->len;
@@ -200,34 +257,32 @@ begin_print (GtkPrintOperation *operation,
                         int mid;
                         mid = length / 2 + length % 2;
                         for (i = 0; i < mid; i++) {
-                                char *unit;
+                                g_autofree char *unit = NULL;
 
                                 g_string_append (s, "\n");
 
                                 unit = gr_ingredients_list_scale_unit (ingredients, segs[j], ings[i], 1, 1);
                                 g_string_append (s, unit);
-                                g_free (unit);
-                                g_string_append (s, " ");
+                                g_clear_pointer (&unit, g_free);
+                                g_string_append (s, "\t");
                                 g_string_append (s, ings[i]);
                                 g_string_append (s, "\t");
                                 if (mid + i < length) {
                                         unit = gr_ingredients_list_scale_unit (ingredients, segs[j], ings[mid + i], 1, 1);
                                         g_string_append (s, unit);
-                                        g_free (unit);
-                                        g_string_append (s, " ");
+                                        g_string_append (s, "\t");
                                         g_string_append (s, ings[mid + i]);
                                 }
                         }
                 }
                 else {
                         for (i = 0; i < length; i++) {
-                                char *unit;
+                                g_autofree char *unit = NULL;
 
                                 g_string_append (s, "\n");
                                 unit = gr_ingredients_list_scale_unit (ingredients, segs[j], ings[i], 1, 1);
                                 g_string_append (s, unit);
-                                g_free (unit);
-                                g_string_append (s, " ");
+                                g_string_append (s, "\t");
                                 g_string_append (s, ings[i]);
                         }
                 }
