@@ -26,10 +26,21 @@
 #include <gtk/gtk.h>
 
 #include "gr-recipe.h"
-#include "gr-images.h"
+#include "gr-recipe-store.h"
+#include "gr-image.h"
 #include "gr-utils.h"
 #include "types.h"
 
+/* Note:
+ *
+ *  If any fields are added to a recipe, there are several places
+ *  that need to be kept in sync:
+ *  - save_recipes() in gr-recipe-store.c
+ *  - load_recipes() in gr-recipe-store.c
+ *  - the GrRecipeExporter code
+ *  - the GrRecipeImporter code
+ */
+
 struct _GrRecipe
 {
         GObject parent_instance;
@@ -38,7 +49,7 @@ struct _GrRecipe
         char *name;
         char *author;
         char *description;
-        GArray *images;
+        GPtrArray *images;
         int default_image;
 
         char *cuisine;
@@ -98,6 +109,8 @@ enum {
         N_PROPS
 };
 
+static char * gr_recipe_get_chef_fullname (GrRecipe *self);
+
 static void
 gr_recipe_finalize (GObject *object)
 {
@@ -115,7 +128,7 @@ gr_recipe_finalize (GObject *object)
         g_free (self->ingredients);
         g_free (self->instructions);
         g_free (self->notes);
-        g_array_free (self->images, TRUE);
+        g_ptr_array_unref (self->images);
 
         g_free (self->cf_name);
         g_free (self->cf_description);
@@ -229,21 +242,22 @@ gr_recipe_get_property (GObject    *object,
         }
 }
 
-static void
-set_images (GrRecipe *self,
-            GArray   *images)
+static char *
+gr_recipe_get_chef_fullname (GrRecipe *self)
 {
-        int i;
+	GrRecipeStore *store;
+	g_autoptr(GrChef) chef = NULL;
 
-        g_array_remove_range (self->images, 0, self->images->len);
-        for (i = 0; i < images->len; i++) {
-                GrImage *ri = &g_array_index (images, GrImage, i);
-                g_array_append_vals (self->images, ri, 1);
-                ri = &g_array_index (self->images, GrImage, i);
-                ri->path = g_strdup (ri->path);
-        }
+	store = gr_recipe_store_get ();
+	chef = gr_recipe_store_get_chef (store, self->author);
+	if (chef) {
+		const char *fullname = gr_chef_get_fullname (chef);
 
-        g_object_notify (G_OBJECT (self), "images");
+                if (fullname)
+			return g_utf8_casefold (fullname, -1);
+	}
+
+        return NULL;
 }
 
 static void
@@ -293,7 +307,9 @@ gr_recipe_set_property (GObject      *object,
                 break;
 
         case PROP_IMAGES:
-                set_images (self, (GArray *) g_value_get_boxed (value));
+                if (self->images)
+                        g_ptr_array_unref (self->images);
+                self->images = g_ptr_array_ref ((GPtrArray *) g_value_get_boxed (value));
                 break;
 
         case PROP_DEFAULT_IMAGE:
@@ -423,8 +439,8 @@ gr_recipe_class_init (GrRecipeClass *klass)
         g_object_class_install_property (object_class, PROP_DESCRIPTION, pspec);
 
         pspec = g_param_spec_boxed ("images", NULL, NULL,
-                                     G_TYPE_ARRAY,
-                                     G_PARAM_READWRITE);
+                                    G_TYPE_PTR_ARRAY,
+                                    G_PARAM_READWRITE);
         g_object_class_install_property (object_class, PROP_IMAGES, pspec);
 
         pspec = g_param_spec_int ("default-image", NULL, NULL,
@@ -685,12 +701,21 @@ gr_recipe_is_contributed (GrRecipe *recipe)
         return recipe->contributed;
 }
 
+GPtrArray *
+gr_recipe_get_images (GrRecipe *recipe)
+{
+        return recipe->images;
+}
+
 /* terms are assumed to be g_utf8_casefold'ed where appropriate */
 gboolean
 gr_recipe_matches (GrRecipe    *recipe,
                    const char **terms)
 {
         int i;
+        g_autofree char *cf_fullname = NULL;
+
+        cf_fullname = gr_recipe_get_chef_fullname (recipe);
 
         for (i = 0; terms[i]; i++) {
                 if (g_str_has_prefix (terms[i], "i+:")) {
@@ -781,6 +806,9 @@ gr_recipe_matches (GrRecipe    *recipe,
                 if (recipe->cf_ingredients && strstr (recipe->cf_ingredients, terms[i]) != NULL)
                         continue;
 
+                if (cf_fullname && strstr (cf_fullname, terms[i]) != NULL)
+                        continue;
+
                 return FALSE;
         }
 
